@@ -16,13 +16,15 @@ import time
 from MoinMoin import log
 logging = log.getLogger(__name__)
 
-from MoinMoin import config, error, util, wikiutil
+from MoinMoin import config, error, util, wikiutil, web
+from MoinMoin import datastruct
 from MoinMoin.auth import MoinAuth
+import MoinMoin.auth as authmodule
 import MoinMoin.events as events
 from MoinMoin.events import PageChangedEvent, PageRenamedEvent
 from MoinMoin.events import PageDeletedEvent, PageCopiedEvent
 from MoinMoin.events import PageRevertedEvent, FileAttachedEvent
-from MoinMoin import session
+import MoinMoin.web.session
 from MoinMoin.packages import packLine
 from MoinMoin.security import AccessControlList
 from MoinMoin.support.python_compatibility import set
@@ -249,6 +251,11 @@ class ConfigFunctionality(object):
             name = dirname + '_dir'
             if not getattr(self, name, None):
                 setattr(self, name, os.path.abspath(os.path.join(data_dir, dirname)))
+        # directories below cache_dir (using __dirname__ to avoid conflicts)
+        for dirname in ('session', ):
+            name = dirname + '_dir'
+            if not getattr(self, name, None):
+                setattr(self, name, os.path.abspath(os.path.join(self.cache_dir, '__%s__' % dirname)))
 
         # Try to decode certain names which allow unicode
         self._decode()
@@ -275,6 +282,32 @@ class ConfigFunctionality(object):
                      Please change it in your wiki configuration and try again."""
             raise error.ConfigurationError(msg)
 
+        # moin < 1.9 used cookie_lifetime = <float> (but converted it to int) for logged-in users and
+        # anonymous_session_lifetime = <float> or None for anon users
+        # moin >= 1.9 uses cookie_lifetime = (<float>, <float>) - first is anon, second is logged-in
+        if not (isinstance(self.cookie_lifetime, tuple) and len(self.cookie_lifetime) == 2):
+            logging.error("wiki configuration has an invalid setting: " +
+                          "cookie_lifetime = %r" % (self.cookie_lifetime, ))
+            try:
+                anon_lifetime = self.anonymous_session_lifetime
+                logging.warning("wiki configuration has an unsupported setting: " +
+                                "anonymous_session_lifetime = %r - " % anon_lifetime +
+                                "please remove it.")
+                if anon_lifetime is None:
+                    anon_lifetime = 0
+                anon_lifetime = float(anon_lifetime)
+            except:
+                # if anything goes wrong, use default value
+                anon_lifetime = 0
+            try:
+                logged_in_lifetime = int(self.cookie_lifetime)
+            except:
+                # if anything goes wrong, use default value
+                logged_in_lifetime = 12
+            self.cookie_lifetime = (anon_lifetime, logged_in_lifetime)
+            logging.warning("using cookie_lifetime = %r - " % (self.cookie_lifetime, ) +
+                            "please fix your wiki configuration.")
+
         self._loadPluginModule()
 
         # Preparse user dicts
@@ -323,6 +356,7 @@ class ConfigFunctionality(object):
                 if not input in self.auth_login_inputs:
                     self.auth_login_inputs.append(input)
         self.auth_have_login = len(self.auth_login_inputs) > 0
+        self.auth_methods = found_names
 
         # internal dict for plugin `modules' lists
         self._site_plugin_lists = {}
@@ -696,22 +730,37 @@ class DefaultExpression(object):
 # information on the layout of this structure.
 #
 options_no_group_name = {
+  # =========================================================================
+  'attachment_extension': ("Mapping of attachment extensions to actions", None,
+  (
+   ('extensions_mapping',
+       {'.tdraw': {'modify': 'twikidraw'},
+        '.adraw': {'modify': 'anywikidraw'},
+       }, "file extension -> do -> action"),
+  )),
+  # ==========================================================================
+  'datastruct': ('Datastruct settings', None, (
+    ('dicts', lambda cfg, request: datastruct.WikiDicts(request),
+     "function f(cfg, request) that returns a backend which is used to access dicts definitions."),
+    ('groups', lambda cfg, request: datastruct.WikiGroups(request),
+     "function f(cfg, request) that returns a backend which is used to access groups definitions."),
+  )),
   # ==========================================================================
   'session': ('Session settings', "Session-related settings, see HelpOnSessions.", (
-    ('session_handler', DefaultExpression('session.DefaultSessionHandler()'),
-     "See HelpOnSessions."),
-    ('session_id_handler', DefaultExpression('session.MoinCookieSessionIDHandler()'),
-     "Only used by the DefaultSessionHandler, see HelpOnSessions."),
+    ('session_service', DefaultExpression('web.session.FileSessionService()'),
+     "The session service."),
+    ('cookie_name', None,
+     'The variable part of the session cookie name. (None = determine from URL, siteidmagic = use siteid, any other string = use that)'),
     ('cookie_secure', None,
      'Use secure cookie. (None = auto-enable secure cookie for https, True = ever use secure cookie, False = never use secure cookie).'),
+    ('cookie_httponly', False,
+     'Use a httponly cookie that can only be used by the server, not by clientside scripts.'),
     ('cookie_domain', None,
      'Domain used in the session cookie. (None = do not specify domain).'),
     ('cookie_path', None,
-     'Path used in the session cookie (None = auto-detect).'),
-    ('cookie_lifetime', 12,
-     'Session lifetime [h] of logged-in users (see HelpOnSessions for details).'),
-    ('anonymous_session_lifetime', None,
-     'Session lifetime [h] of users who are not logged in (None = disable anon sessions).'),
+     'Path used in the session cookie (None = auto-detect). Please only set if you know exactly what you are doing.'),
+    ('cookie_lifetime', (0, 12),
+     'Session lifetime [h] of (anonymous, logged-in) users (see HelpOnSessions for details).'),
   )),
   # ==========================================================================
   'auth': ('Authentication / Authorization / Security settings', None, (
@@ -719,7 +768,7 @@ options_no_group_name = {
      "List of trusted user names with wiki system administration super powers (not to be confused with ACL admin rights!). Used for e.g. software installation, language installation via SystemPagesSetup and more. See also HelpOnSuperUser."),
     ('auth', DefaultExpression('[MoinAuth()]'),
      "list of auth objects, to be called in this order (see HelpOnAuthentication)"),
-    ('auth_methods_trusted', ['http', 'xmlrpc_applytoken'],
+    ('auth_methods_trusted', ['http', 'given', 'xmlrpc_applytoken'], # Note: 'http' auth method is currently just a redirect to 'given'
      'authentication methods for which users should be included in the special "Trusted" ACL group.'),
     ('secrets', None, """Either a long shared secret string used for multiple purposes or a dict {"purpose": "longsecretstring", ...} for setting up different shared secrets for different purposes. If you don't setup own secret(s), a secret string will be auto-generated from other config settings."""),
     ('DesktopEdition',
@@ -761,14 +810,14 @@ options_no_group_name = {
         'rss_rc': (1, 60),
         # The following actions are often used for images - to avoid pages with lots of images
         # (like photo galleries) triggering surge protection, we assign rather high limits:
-        'AttachFile': (90, 60),
+        'AttachFile': (300, 30),
         'cache': (600, 30), # cache action is very cheap/efficient
      },
      "Surge protection tries to deny clients causing too much load/traffic, see HelpOnConfiguration/SurgeProtection."),
     ('surge_lockout_time', 3600, "time [s] someone gets locked out when ignoring the warnings"),
 
     ('textchas', None,
-     "Spam protection setup using site-specific questions/answers, see HelpOnTextChas."),
+     "Spam protection setup using site-specific questions/answers, see HelpOnSpam."),
     ('textchas_disabled_group', None,
      "Name of a group of trusted users who do not get asked !TextCha questions."),
 
@@ -804,7 +853,7 @@ options_no_group_name = {
     ('navi_bar', [u'RecentChanges', u'FindPage', u'HelpContents', ],
      'Most important page names. Users can add more names in their quick links in user preferences. To link to URL, use `u"[[url|link title]]"`, to use a shortened name for long page name, use `u"[[LongLongPageName|title]]"`. [list of Unicode strings]'),
 
-    ('theme_default', 'modern',
+    ('theme_default', 'modernized',
      "the name of the theme that is used by default (see HelpOnThemes)"),
     ('theme_force', False,
      "if True, do not allow to change the theme"),
@@ -837,7 +886,8 @@ options_no_group_name = {
 
     ('edit_bar', ['Edit', 'Comments', 'Discussion', 'Info', 'Subscribe', 'Quicklink', 'Attachments', 'ActionsMenu'],
      'list of edit bar entries'),
-    ('history_count', (100, 200), "number of revisions shown for info/history action (default_count_shown, max_count_shown)"),
+    ('history_count', (100, 200, 5, 10, 25, 50), "Number of revisions shown for info/history action (default_count_shown, max_count_shown, [other values shown as page size choices]). At least first two values (default and maximum) should be provided. If additional values are provided, user will be able to change number of items per page in the UI."),
+    ('history_paging', True, "Enable paging functionality for info action's history display."),
 
     ('show_hosts', True,
      "if True, show host names and IPs. Set to False to hide them."),
@@ -849,8 +899,7 @@ options_no_group_name = {
      'show section numbers in headings by default'),
     ('show_timings', False, "show some timing values at bottom of a page"),
     ('show_version', False, "show moin's version at the bottom of a page"),
-    ('traceback_show', True,
-     "if True, show debug tracebacks to users when moin crashes"),
+    ('show_rename_redirect', False, "if True, offer creation of redirect pages when renaming wiki pages"),
 
     ('packagepages_actions_excluded',
      ['setthemename',  # related to questionable theme stuff, see below
@@ -912,6 +961,7 @@ options_no_group_name = {
     ('data_dir', './data/', "Path to the data directory containing your (locally made) wiki pages."),
     ('data_underlay_dir', './underlay/', "Path to the underlay directory containing distribution system and help pages."),
     ('cache_dir', None, "Directory for caching, by default computed from `data_dir`/cache."),
+    ('session_dir', None, "Directory for session storage, by default computed to be `cache_dir`/__session__."),
     ('user_dir', None, "Directory for user storage, by default computed to be `data_dir`/user."),
     ('plugin_dir', None, "Plugin directory, by default computed to be `data_dir`/plugin."),
     ('plugin_dirs', [], "Additional plugin directories."),
@@ -944,8 +994,8 @@ options_no_group_name = {
   )),
   # ==========================================================================
   'pages': ('Special page names', None, (
-    ('page_front_page', u'HelpOnLanguages',
-     "Name of the front page. We don't expect you to keep the default. Just read HelpOnLanguages in case you're wondering... [Unicode]"),
+    ('page_front_page', u'LanguageSetup',
+     "Name of the front page. We don't expect you to keep the default. Just read LanguageSetup in case you're wondering... [Unicode]"),
 
     # the following regexes should match the complete name when used in free text
     # the group 'all' shall match all, while the group 'key' shall match the key only
@@ -1142,7 +1192,7 @@ options = {
       ('checkbox_defaults',
        {
         'mailto_author': 0,
-        'edit_on_doubleclick': 0,
+        'edit_on_doubleclick': 1,
         'remember_last_visit': 0,
         'show_comments': 0,
         'show_nonexist_qm': False,
@@ -1194,6 +1244,12 @@ options = {
        "User object attributes that are not persisted to permanent storage (internal use)."),
     )),
 
+    'openidrp': ('OpenID Relying Party',
+        'These settings control the built-in OpenID Relying Party (client).',
+    (
+      ('allowed_op', [], "List of forced providers"),
+    )),
+
     'openid_server': ('OpenID Server',
         'These settings control the built-in OpenID Identity Provider (server).',
     (
