@@ -14,7 +14,7 @@ from cStringIO import StringIO
 from MoinMoin.converter import text_html_text_moin_wiki as converter
 from MoinMoin.parser.text_moin_wiki import Parser
 from MoinMoin.formatter.text_gedit import Formatter
-from MoinMoin.request import Clock
+from MoinMoin.util.clock import Clock
 from MoinMoin.error import ConvertError
 
 convert = converter.convert
